const webpack = require('webpack');
const path = require('path');
let env = require('yargs').argv.env; // use --env with webpack 2
const TerserPlugin = require('terser-webpack-plugin');
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;
const {libraryBaseName} = require('./config/config.json');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CopyPlugin = require('copy-webpack-plugin');

const sdkVersion = JSON.stringify(require("./package.json").version);

// In order to handle multiple env
// convert env to an array of env(s)
env = (function (env) {
    if (Array.isArray(env)) {
        return env;
    }
    return [env]
})(env)

const baseConfig = {
    mode: isProd() ? 'production' : 'development',
    entry: __dirname + '/src/index.ts',
    devtool: isDev() ? 'source-map' : false,
    module: {
        rules: [
            {
                test: /\.ts$/,
                enforce: 'pre',
                exclude: /(node_modules|bower_components)/,
                use: {
                    loader: 'tslint-loader',
                    options: {
                        configFile: './tslint.json',
                        failOnHint: true
                    },
                },
            },
            {
                test: /\.tsx?$/,
                use: {
                    loader: 'ts-loader',
                },
                exclude: /(node_modules|bower_components)/
            },
            {
                test: /\.jsx?$/,
                use: {
                    loader: 'babel-loader',
                },
                exclude: /(node_modules|bower_components)/
            },
        ]
    },
    resolve: {
        modules: [path.resolve('./node_modules'), path.resolve('./src')],
        extensions: ['.json', '.js', '.ts', '.tsx', '.json'],
        alias: {
            '@': path.resolve(__dirname, "./src/"),
            'config': path.resolve(__dirname, 'config/'),
            'assets': path.resolve(__dirname, 'src/assets/'),
        }
    },
    optimization: {
        minimize: (isProd()),
        minimizer: [new TerserPlugin()],
    },
    plugins: [
        // new BundleAnalyzerPlugin(),
        new webpack.DefinePlugin({
            "IS_HTTP_MODE": isHttpMode(),
            "IS_HTTP_WINDOW_MODE": isHttpWindowMode(),
        })
    ],
};

/**
 * In order for our webpush SDK to work, we should have 4 different files,
 * 2 different SDK builds, a Service Worker build, and a Service Worker fetcher build
 *
 * One of the following two builds will be used as our SDK build,
 * It has all API functions such as init, subscribe, unsubscribe, captureEvent, ...
 */

/**
 * 1.
 * This is the build used with npm [CommonJS Style Imports], or anywhere the user uses
 * something like const Pushe = require('pushe-web'), or import Pushe from 'pushe-web'
 */
const sdkCommonJSConfig = {
    ...baseConfig,
    entry: __dirname + '/src/index-npm.ts',
    output: {
        path: __dirname + '/lib',
        filename: `${libraryBaseName.toLowerCase()}web-npm.js`,
        // library: libraryBaseName, // Do not use library key in here
        libraryTarget: 'commonjs2'
    },
    plugins: [
        ...baseConfig.plugins,
        new webpack.DefinePlugin({
            "RUNTIME_ENV": JSON.stringify("npm"),
            VERSION: sdkVersion,
        })
    ]
}


/**
 * 2.
 * This is the build used with conventional websites, used like:
 * <script type... src='pusheweb.js'
 */
const sdkWindowConfig = {
    ...baseConfig,
    output: {
        path: __dirname + '/lib',
        filename: `${libraryBaseName.toLowerCase()}web.js`,
        library: libraryBaseName,
        libraryTarget: 'window'
    },
    devServer: {
        contentBase: path.resolve(__dirname, 'dist'),
        publicPath: '/',
        port: 8000,
        open: true,
        injectClient: false, // This is really needed
    },
    plugins: [
        ...baseConfig.plugins,
        new HtmlWebpackPlugin({
            template: path.resolve(__dirname, 'demo', 'index.html'),
            inject: 'head',
        }),
        new CopyPlugin({
            patterns: [
                './demo/pushe-sw.js',
                './demo/http.html'
            ]
        }),
        new webpack.DefinePlugin({
            "RUNTIME_ENV": JSON.stringify("window"),
            VERSION: sdkVersion,
        }),
    ]
}


/**
 * This is the ServiceWorker build, an extra pushe-sw.js file should be provided to
 * the user, which only has one line of code, it fetches the following file as
 * the Service Worker,
 * This file compiles to pusheweb-sw-contents.js
 * Other file is probably like: importScripts('https://static.pushe.co/sdks/pusheweb-sw-contents.js');
 */
const serviceWorkerConfig = {
    ...baseConfig,
    output: {
        path: __dirname + '/lib',
        filename: `${libraryBaseName.toLowerCase()}web-sw.js`,
        library: libraryBaseName,
        libraryTarget: 'var'
    },
    plugins: [
        ...baseConfig.plugins,
        new webpack.DefinePlugin({
            "RUNTIME_ENV": JSON.stringify("sw"),
            VERSION: sdkVersion,
        })
    ]
}


function isDev() {
    return env[0] === 'dev';
}

function isProd() {
    return env[0] === 'build'
}

// Only for development and http mode
function isHttpMode() {
    if (env.length > 1 && env[0] === 'dev') {
        return env[1] === 'http'
    }
    return false;
}

// Only for development and http mode window (for testing subdomain window)
function isHttpWindowMode() {
    if (env.length > 1 && env[0] === 'dev') {
        return env[1] === 'httpwindow'
    }
    return false;
}

module.exports = [sdkCommonJSConfig, sdkWindowConfig, serviceWorkerConfig];
